@@ -16,14 +16,14 @@ class OTLClassCreator(AbstractDatatypeCreator):
     def __init__(self, oslo_collector: OSLOCollector, geo_a_collector: GeometrieArtefactCollector = None):
         super().__init__(oslo_collector)
         logging.info("Created an instance of OTLClassCreator")
-        self.osloCollector = oslo_collector
+        self.oslo_collector = oslo_collector
         self.geoACollector = geo_a_collector
         self.geometry_types = []
 
         if geo_a_collector is not None:
             gip = GeometrieInheritanceProcessor(classes=oslo_collector.classes,
                                                 geometrie_types=self.geoACollector.geometrie_types,
-                                                inheritances=self.osloCollector.inheritances)
+                                                inheritances=self.oslo_collector.inheritances)
             self.geometry_types = gip.process_inheritances()
 
     def create_blocks_to_write_from_classes(self, oslo_class: OSLOClass, model_location='') -> [str]:
@@ -42,8 +42,8 @@ class OTLClassCreator(AbstractDatatypeCreator):
         return self.create_block_from_class(oslo_class, model_location)
 
     def create_block_from_class(self, oslo_class: OSLOClass, model_location: str = '') -> [str]:
-        attributen = self.osloCollector.find_attributes_by_class(oslo_class)
-        inheritances = self.osloCollector.find_inheritances_by_class(oslo_class)
+        attributen = self.oslo_collector.find_attributes_by_class(oslo_class)
+        inheritances = self.oslo_collector.find_inheritances_by_class(oslo_class)
         list_of_geometry_types = self.get_geometry_types_from_uri(oslo_class.objectUri)
 
         if oslo_class.objectUri in ['https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#AIMObject',
@@ -132,6 +132,8 @@ class OTLClassCreator(AbstractDatatypeCreator):
         datablock.append('')
         datablock.append('')
         datablock.append(f'# Generated with {self.__class__.__name__}. To modify: extend, do not edit')
+
+        inheritances = self.sort_inheritances_for_relation_interactor_priority(inheritances)
         datablock.append(self.get_class_line_from_class_and_inheritances(oslo_class=oslo_class,
                                                                          inheritances=inheritances,
                                                                          geometry_types=list_of_geometry_types))
@@ -152,7 +154,7 @@ class OTLClassCreator(AbstractDatatypeCreator):
             datablock.append('        super().__init__()')
             datablock.append('')
         elif len(inheritances) + len(list_of_geometry_types) > 1:
-            for inheritance in sorted(inheritances, key=lambda a: a.base_name):
+            for inheritance in inheritances:
                 datablock.append(f'        {inheritance.base_name}.__init__(self)')
             for geo_type in sorted(list_of_geometry_types):
                 datablock.append(f'        {geo_type}.__init__(self)')
@@ -208,7 +210,7 @@ class OTLClassCreator(AbstractDatatypeCreator):
         return geom_types
 
     def add_relations_to_datablock(self, datablock: [str], object_uri: str) -> None:
-        relations = self.osloCollector.find_outgoing_relations(object_uri)
+        relations = self.oslo_collector.find_outgoing_relations(object_uri)
         if len(relations) == 0:
             return
 
@@ -219,3 +221,34 @@ class OTLClassCreator(AbstractDatatypeCreator):
             datablock.append(
                 f"        self.add_valid_relation(relation='{relation.objectUri}', target='{relation.doel_uri}'{deprecated})")
         datablock.append('')
+
+    relation_interactor_uris = ['https://wegenenverkeer.data.vlaanderen.be/ns/abstracten#AbstracteAanvullendeGeometrie',
+                                'https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#AIMObject',
+                                'https://wegenenverkeer.data.vlaanderen.be/ns/implementatieelement#Derdenobject',
+                                'http://purl.org/dc/terms/Agent']
+
+    def search_recursive_inheritance_for_relation_interactor(self, base_uri: str) -> bool:
+        inheritances = self.oslo_collector.find_inheritances_by_class_uri(base_uri)
+        for inheritance in inheritances:
+            if inheritance.base_uri in self.relation_interactor_uris:
+                return True
+            if self.search_recursive_inheritance_for_relation_interactor(inheritance.base_uri):
+                return True
+        return False
+
+    def sort_inheritances_for_relation_interactor_priority(self, inheritances):
+        first_good_candidate = None
+
+        for inheritance in inheritances:
+            if first_good_candidate is not None:
+                break
+            if self.search_recursive_inheritance_for_relation_interactor(inheritance.base_uri):
+                first_good_candidate = inheritance
+                break
+
+        if first_good_candidate is None:
+            return inheritances
+
+        inheritances.remove(first_good_candidate)
+        inheritances.insert(0, first_good_candidate)
+        return inheritances
